#ifndef __DISTINTERFACE_H__
#define __DISTINTERFACE_H__

#include "config.h"
#include "monitor.h"

#define MAX_ARGS 24             /* max # args allowed to send to distiller */
#define MAX_ARG_STRING 80       /* max length of a string-type arg */
#include <stdlib.h> 
#include <string.h>

typedef enum ArgumentType {
  typeInt, 
  typeString, 
  typeDouble
} ArgumentType;


typedef enum DistillerStatus {
  distOk=0,
  distNoCacheManager,       /*the cache manager seems to have disappeared*/
  distDistillerNotFound,    /*No distiller found; or PTM does not exist*/
  distSendError,            /*Error occurred while sending data*/
  distReplyTimeout,         /*Timed out while waiting for reply*/
  distOutOfLocalMemory,     /*Proxy front-end ran out of memory*/
  distConnectionBroken,     /*Connection to the distiller was broken*/
  distLaunchTimeout,        /*Timed out while asking PTM for a new distiller*/

  distFatalError
} DistillerStatus;


typedef struct DistillationCost {
  UINT32 estimatedTime_ms; /* currently always set to 1 => load metric is
                              the same as queue length */
} DistillationCost;


typedef struct Argument {
#ifdef __cplusplus
  Argument() : type(typeInt), id(0) { };
#endif

  ArgumentType type;
  UINT32       id;
  union Union {
#ifdef __cplusplus
    Union() : arg_int(0) { };
#endif

    INT32 arg_int;
    char   arg_string[MAX_ARG_STRING];
    double arg_double;
  } a;
} Argument;



#define MAX_MIMETYPE 50

typedef struct DistillerData {
#ifdef __cplusplus
  DistillerData() : data(0), length(0) {
    strcpy(mimeType, "");
  }
#endif

  void   *data;
  UINT32 length;
  char   mimeType[MAX_MIMETYPE];
} DistillerData;


typedef DistillerData DistillerInput;
typedef DistillerData DistillerOutput;



#define SET_ARG_ID(arg,i)      ((arg).id = i)
#define SET_ARG_STRING(arg,s)   { (arg).type = typeString; \
                                  strncpy((arg).a.arg_string,s,MAX_ARG_STRING); }
#define SET_ARG_INT(arg,i)      { (arg).type = typeInt; (arg).a.arg_int = i; }
#define SET_ARG_DOUBLE(arg,d)   { (arg).type=typeDouble; (arg).a.arg_double=d; }

#define ARG_TYPE(arg)     ((arg).type)
#define ARG_ID(arg)       ((arg).id)
#define ARG_STRING(arg)   ((arg).type == typeString? (arg).a.arg_string: "")
#define ARG_INT(arg)      ((arg).type == typeInt? (arg).a.arg_int: 0)
#define ARG_DOUBLE(arg)   ((arg).type == typeDouble? (arg).a.arg_double: 0.0)

#define _ARG_STRING(arg)  ((arg).a.arg_string)
#define _ARG_INT(arg)     ((arg).a.arg_int)
#define _ARG_DOUBLE(arg)  ((arg).a.arg_double)

#ifdef __cplusplus
extern "C" {
#endif


Argument *getArgumentFromId(Argument *args, int numberOfArgs, UINT32 id);
#define getArgumentFromIdInList(al,id)  \
                getArgumentFromId((al)->arg,(al)->nargs,(id))
/*
 * returns a pointer to the argument corresponding to id
 * NULL, if no such arg can be found
 */


DistillerStatus DistillerInit();
/*
 * Front-end calls this function to perform startup stuff
 * Returns distOk on success, distFatalError on failure
 */

void DistillerCleanup();
/*
 * Front-end calls this function before exiting
 */


DistillerStatus
DistillerMain(Argument *args, int numberOfArgs,
	      DistillerInput *input, DistillerOutput *output,
	      gm_Bool *freeOutputBuffer);
/* 
 * the freeOutputBuffer arg indicates whether the distiller front-end should
 * free the output buffer or not. Valid values are gm_True and gm_False
 */


DistillerStatus
ComputeDistillationCost(Argument *args, int numberOfArgs,
			DistillerInput *input, DistillationCost *cost);
/*
 * This function should calculate an estimate of the time to perform a
 * distillation.
 * The function returns distOk on success.
 */

#ifndef DEBUGGING
void *DistillerMalloc(size_t size);
void *DistillerRealloc(void *ptr, size_t size);
/*
 * Use these function to malloc memory for the output buffer
 */

void DistillerFree(void *pointer);
/*
 * Use this function to free memory allocated using DistillerMalloc
 * DO NOT free the output buffer if the "freeOutputBuffer" return arg is 
 * set to gm_True inside DistillerMain.
 */
#else  /* DEBUGGING */
#define DistillerMalloc malloc
#define DistillerFree   free
#define DistillerRealloc realloc
#endif


MonitorClientID DistillerGetMonitorClientID();


#ifdef __cplusplus
}
#endif

#endif 
